package CPU;

/**
 * This is used to communicate with the Java monitor for TRAP calls. It makes some unused 68000 instructions return to the emulator.
 * This class is used to communicate with the Java monitor for TRAP calls.
 * 
 * The actual "illegal instructions" that are used are defined as constants in the Types object.
 * 
 * v. 1.02 instructions are no longer negative; "traps" up to TRAPf; add "trap" names to disassembled instruction.
 * v. 1.01 use Aline instructions for "traps" up to TRAPd right now.
 * 
 * @author and copyright (c) 2012 -2023 Wolfgang Lenerz.
 * 
 */
    
public class JavaComm implements CPU.InstructionSet
{
    private final int base=smsqmulator.Types.RETURN_BASE;                     // unused instructions by the MC 680000

    /**
     * Registers the instruction with the CPU.
     * 
     * @param cpu2 the cpu with which to register.
     */
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        for (int i=0;i<17;i++)
        {
            cpu2.addInstruction(this.base+i, new CPU.Instruction() 
            {
                private final CPU.MC68000Cpu cpu=cpu2;
                
                @Override
                public void execute(int opcode,CPU.MC68000Cpu cpu)
                {
                    this.cpu.trapDispatcher.dispatchTrap(opcode,cpu); 
                }
                @Override
                public CPU.DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                {
                        return new CPU.DisassembledInstruction(address, opcode, "JavaCom  TRAP #" + (opcode-base)+smsqmulator.Types.TRAP_INFO[(opcode-base)]);
                }
            });
        }
        
        for (int i=0xb00;i<0xb20;i++)                           // replacement ari operations
        {
            cpu2.addInstruction(this.base+i, new CPU.Instruction() 
            {
                private final CPU.MC68000Cpu cpu=cpu2;
                
                @Override
                public void execute(int opcode,CPU.MC68000Cpu cpu)
                {                          
                 //   if (this.cpu.trapDispatcher!=null)
                    this.cpu.trapDispatcher.dispatchTrap(opcode,cpu); 
                }
                @Override
                public CPU.DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                {
                    return new CPU.DisassembledInstruction(address, opcode, "JavaCom  ARIOP " + (base+0xb00-opcode));
                }
            });
        }
        
    }
}

