package CPU.instructions.ADD2EA;
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;
import CPU.Size;

/**
 * The ADD instruction where the source is Dn and the destination is -(An).
 *  
 * @author and copyright (c) 2013 wolfgang lenerz
 * 
 *  1101dddooommmrrr
 *  
 *  where   
 *          ddd is the destination register
 *          ooo is the opmode : 100 = byte, 101 = word , 110 =long
 *          mmm = ea mode =100, 
 *          rrr = source register
 * @version
 *  1.01 d must be cut to size (.b or .w) before testing whether it's 0.
 * 
 * Based on code by Tony Headford, see his licence in accompanying file.
 */

public class ADD_MinusAn implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        int base;
        CPU.Instruction i;

        for(int sz = 0; sz < 3; sz++)                       
        {
            if(sz == 0)
            {
                base = 0xd120;
                i = new CPU.Instruction()    // byte sized add
                {
                    
                    
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        boolean Sm, Rm,Dm;
                        int reg=opcode&7;
                        int s=cpu.data_regs[(opcode>>9)&7]&0xff;
                        if ((s & 0x80)!=0)
                        {
                            s|=0xffffff00;     
                            Sm=true;
                        }
                        else
                        {
                            Sm=false;
                        }
                        if (reg==7)
                        {
                            cpu.addr_regs[reg]-=2;
                        }
                        else
                        {
                            cpu.addr_regs[reg]--;
                        }
                       
                        int d=cpu.readMemoryByteSigned(cpu.addr_regs[reg]);
                        Dm=d<0;
                        
                        d+=s;
                        Rm = (d & 0x80)!=0;
                        cpu.writeMemoryByte(cpu.addr_regs[reg],d); 
                        cpu.reg_sr&=0xffe0;            // all flags 0
                       
                        if ((d&0xff) == 0)
                        {
                            cpu.reg_sr+=4;             // set Z flag
                        }
                        else if (Rm)
                        {
                            cpu.reg_sr+=8;             // set N flag
                        }
                       
                        if ((Sm && Dm && !Rm) || (!Sm && !Dm && Rm))
                        {
                            cpu.reg_sr+=2;              // set V flag
                        }
				
                        if ((Sm && Dm) || (!Rm && Dm) || (Sm && !Rm))
                        {
                            cpu.reg_sr+=17;              // set X and C flags
                        }
                    }
                    @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Byte,cpu);
                    }

                };
            }
            else if(sz == 1)
            {
                base = 0xd160;
                i = new CPU.Instruction()    // word sized add
                {
                    
                    
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        boolean Sm, Rm,Dm;
                       
                        int reg=opcode&7;
                        int s=cpu.data_regs[(opcode>>9)&7]&0xffff;
                        if ((s & 0x8000)!=0)
                        {
                            s|=0xffff0000;     
                            Sm=true;
                        }
                        else
                        {
                            Sm=false;
                        }
                        
                        cpu.addr_regs[reg]-=2;
                        int d=cpu.readMemoryWordSigned(cpu.addr_regs[reg]);
                        Dm=d<0;
                        
                        d+=s;
                        Rm = (d & 0x8000)!=0;
                        cpu.writeMemoryWord(cpu.addr_regs[reg],d);  
                        cpu.reg_sr&=0xffe0;            // all flags 0
                       
                        if ((d&0xffff) == 0)
                        {
                            cpu.reg_sr+=4;             // set Z flag
                        }
                        else if (Rm)
                        {
                            cpu.reg_sr+=8;             // set N flag
                        }
                       
                        if ((Sm && Dm && !Rm) || (!Sm && !Dm && Rm))
                        {
                            cpu.reg_sr+=2;              // set V flag
                        }
				
                        if ((Sm && Dm) || (!Rm && Dm) || (Sm && !Rm))
                        {
                            cpu.reg_sr+=17;              // set X and C flags
                        }
                    }
                    @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Word,cpu);
                    }

                };
            }
            else
            {
                base = 0xd1a0;
                i = new CPU.Instruction()    // long sized add
                {
                    
                    
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {                   
                        int reg=opcode&7;
                        int s=cpu.data_regs[(opcode>>9)&7];
                        boolean Sm=s<0;
                        cpu.addr_regs[reg]-=4;
                        int d=cpu.readMemoryLong(cpu.addr_regs[reg]);
                        boolean Dm=d<0;
                        d+=s;
                        boolean Rm=d<0;
                        cpu.writeMemoryLong(cpu.addr_regs[reg],d); 
                        cpu.reg_sr &= 0xffe0;            // all flags 0
                       // if (cpu.data_regs[reg] == 0)
                        if (d==0)
                        {
                            cpu.reg_sr+=4;             // set Z flag
                        }
                        else if (Rm)
                        {
                            cpu.reg_sr+=8;             // set N flag
                        }
                       
                        if ((Sm && Dm && !Rm) || (!Sm && !Dm && Rm))
                        {
                            cpu.reg_sr+=2;             // set V flag
                        }
				
                        if ((Sm && Dm) || (!Rm && Dm) || (Sm && !Rm))
                        {
                            cpu.reg_sr+=17;            // set X and C flags
                        }
                    }
                    @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Long,cpu);
                    }
                };
            }
            
            for(int reg = 0; reg < 8; reg++)
            {
                for(int ea_reg = 0; ea_reg < 8; ea_reg++)
                {
                    cpu2.addInstruction(base + (reg << 9) + ea_reg, i);
                }                    
            }
        }
    }
    
    /**
     * Disassemble the instruction.
     * 
     * @param address   address of op in memory
     * @param opcode    MC68000 opcode of operation
     * @param sz        size (.b, .w, .l)
     * @param cpu       cpu used 
     * 
     * @return the DisassembledInstruction
     */
    protected final DisassembledInstruction disassembleOp(int address, int opcode, Size sz,CPU.MC68000Cpu cpu)
    {
        DisassembledOperand src = new DisassembledOperand("d" + ((opcode >> 9) & 0x07));
        DisassembledOperand dst = cpu.disassembleDstEA(address + 2, (opcode >> 3) & 0x07, (opcode & 0x07), sz);
        return new DisassembledInstruction(address, opcode, "add" + sz.ext(), src, dst);
    }
}
