package CPU.instructions;
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;
import CPU.Size;
/**
 * The ANDI_TO_CCR instruction in all of its variants.
 *  fedcba9876543210
 *  0x023c
 * @author and Copyright (c) for my code Wolfgang Lenerz 2013
 * 
 * Based on code by Tony Headford, see his licence in accompanying file.
 * @version
 * 1.01 only set the BYTE of the SR!
 * 1.02 correct disassembly (only byte, no size)
 * 
 */
public class ANDI_CCR implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        CPU.Instruction i= new CPU.Instruction()   
        {
            @Override
            public final void execute(int opcode,CPU.MC68000Cpu cpu)
            {
                int val=cpu.readMemoryWordPCInc()&0x1f;
                val&=(cpu.reg_sr&0xff);
                cpu.reg_sr&=0xff00;
                cpu.reg_sr|=val;
            }
            
            @Override
            public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
            {
                int imm_bytes;
		int imm;
		String is;
		imm = cpu.readMemoryWord(address + 2)&0xff;
                is = String.format("#$%02x", imm);
                imm_bytes = 2;
		DisassembledOperand src = new DisassembledOperand(is, imm_bytes, imm);
		DisassembledOperand dst = cpu.disassembleDstEA(address + 2 + imm_bytes, (opcode >> 3) & 0x07, (opcode & 0x07), Size.Byte);
		return new DisassembledInstruction(address, opcode, "andi" + Size.Unsized.ext(), src, dst);
            }   
        };
        cpu2.addInstruction(0x23c,i);
    }
}
