package CPU.instructions.JMP;
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;
import CPU.Size;

/**
 * The JMP instruction where source is d16(An).
 * Based on code by Tony Headford, see his licence in accompanying file.
 
 * @author and copyright (c) 2012 wolfgang lenerz
 * 
 *  01001110 = 0x4e
 *  11mmmrrr 
 *  where m = mode =101
 */
public class JMP2 implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        int base= 0x4ee8;
        CPU.Instruction i;
        i = new CPU.Instruction()
        {
            @Override
            public final void execute(int opcode,CPU.MC68000Cpu cpu)
            {
		cpu.pc_reg=((cpu.addr_regs[opcode &7] + cpu.readMemoryWordPCSignedInc()) & CPU.MC68000Cpu.CUT_OFF)/2;
            }
            @Override
            public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
            {
                    return disassembleOp(address, opcode, Size.Long, cpu);
            }
        };

        for(int ea_reg = 0; ea_reg < 8; ea_reg++)
        {
            cpu2.addInstruction(base + ea_reg, i);
        }                           
    }
    protected final DisassembledInstruction disassembleOp(int address, int opcode, Size sz, CPU.MC68000Cpu cpu)
    {
            DisassembledOperand op = cpu.disassembleSrcEA(address + 2, (opcode >> 3) & 0x07, (opcode & 0x07), sz);
            return new DisassembledInstruction(address, opcode, "jmp", op);
    }
}
