
package CPU.instructions.MOVEA;

/**
 * MOVE where source is AN and destination is AN.
 * 
 * 00ssxxx001mmmrrr
 * s = size bit
 * x=dest register
 * m =source mode 001
 * r= source register
 * cpu.pc_reg*2+getDisplacement(cpu);
 */
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;
import CPU.Size;

public class MOVEA2 implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        int base;
        CPU.Instruction i;
        for(int sz = 1; sz < 3; sz++)
        {
            if(sz == 1)
            {
                // move word
                base = 0x3048;
                i = new CPU.Instruction()
                {
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                                int s=cpu.addr_regs[opcode &7]&0xffff;
                                if ((s&0x8000)!=0)
                                    s|=0xffff0000;                                   // sign extend
                                cpu.addr_regs[(opcode>>9)&7]=s;
                    }
                   @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Word, cpu);
                    }
                };
            }
            else
            {
                // move long
                base = 0x2048;
                i = new CPU.Instruction()
                {
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        cpu.addr_regs[(opcode>>9)&7]=cpu.addr_regs[opcode &7];
                    }
                    @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Long, cpu);
                    }
                };
            }
            for(int sea_reg = 0; sea_reg < 8; sea_reg++)
            {
                for(int dea_reg = 0; dea_reg < 8; dea_reg++)
                {
                    cpu2.addInstruction(base + (dea_reg << 9)  + sea_reg, i);
                }
            }
        }
    }

    protected final DisassembledInstruction disassembleOp(int address, int opcode, Size sz, CPU.MC68000Cpu cpu)
    {
            DisassembledOperand src = cpu.disassembleSrcEA(address + 2, (opcode >> 3) & 0x07, (opcode & 0x07), sz);
            DisassembledOperand dst = cpu.disassembleDstEA(address + 2 + src.bytes, (opcode >> 6) & 0x07, (opcode >> 9) & 0x07, sz);

            return new DisassembledInstruction(address, opcode, "move" + sz.ext(), src, dst);
    }
}
