package CPU.instructions.MOVEM;

/**
 * This is the MOVEM from  memory to (AN) instruction.
 * MOVEM.L (A1), D1/D2) : data are moved to regs
 *      d
 * 010011001smmmrrr
 * where :
 * d = direction of transfer = 0
 * s = size : 0=Word 1=Long
 * mmm = mode = 010
 * rrr = register
 * 
 * @author and copyright for my code (c) Wolfgang Lenerz 2013 - 2014.
 * 
 * Based on code by Tony Headford, see his licence in accompanying file
 */


import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;
import CPU.Size;

public class MOVEM2Reg1 implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        int base;
        CPU.Instruction i;

        // reg to mem
        for (int sz = 0; sz < 2; sz++)
        {
            if (sz == 0)
            {
                base = 0x4c90;
                i = new CPU.Instruction()
                {
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        int address=cpu.addr_regs[opcode &7]; // the (an) register = the address we're talking about
                        int reglist = cpu.readMemoryWordPCInc();// the registers to treat
                        int bit = 1;
                        //assumes d0 is first bit
                        for(int n = 0; n < 8; n++)
                        {
                            if((reglist & bit) != 0)
                            {
                                cpu.data_regs[n]=cpu.readMemoryWordSigned(address);
                                address+= 2;
                            }
                            bit <<= 1;
                        }

                        for(int n = 0; n < 8; n++)
                        {
                            if((reglist & bit) != 0)
                            {
                                cpu.addr_regs[n]=cpu.readMemoryWordSigned(address);
                                address += 2;
                            }
                            bit <<= 1;
                        }
                    }
                   @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Word, cpu);
                    }
                };
            }
            else
            {
                base = 0x4cd0;
                i = new CPU.Instruction()
                {
                    @Override
                    public final void execute(int opcode,CPU.MC68000Cpu cpu)
                    {
                        int address = cpu.addr_regs[opcode &7]; // the (an) register = the address we're talking about
                        int reglist = cpu.readMemoryWordPCInc();// the registers to treat
                        int bit = 1;
                        //assumes d0 is first bit
                        for(int n = 0; n < 8; n++)
                        {
                            if((reglist & bit) != 0)
                            {
                                cpu.data_regs[n]=cpu.readMemoryLong(address);
                                address+= 4;
                            }
                            bit <<= 1;
                        }

                        for(int n = 0; n < 8; n++)
                        {
                            if((reglist & bit) != 0)
                            {
                                cpu.addr_regs[n]=cpu.readMemoryLong(address);
                                address += 4;
                            }
                            bit <<= 1;
                        }
                    }
                    @Override
                    public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
                    {
                        return disassembleOp(address, opcode, Size.Long, cpu);
                    }
                };
            }

            for(int ea_reg = 0; ea_reg < 8; ea_reg++)
            {
                cpu2.addInstruction(base + ea_reg, i);
            }
        }
    }   
    public final DisassembledInstruction disassembleOp(int address, int opcode, Size sz,CPU.MC68000Cpu cpu)
    {
        DisassembledOperand src;
        DisassembledOperand dst;
        int mode = (opcode >> 3) & 0x07;
        int reg = (opcode & 0x07);
        int reglist = cpu.readMemoryWord(address + 2);
        boolean reversed = (mode == 4);
        src = cpu.disassembleSrcEA(address + 4, mode, reg, sz);
        dst = new DisassembledOperand(smsqmulator.Helper.regListToString(reglist, reversed), 2, reglist);
        return new DisassembledInstruction(address, opcode, "movem" + sz.ext(), src, dst);
    }
}
