package CPU.instructions;
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;

/**
 * A replacement MOVEQ instruction in all of its variants
 * 
 * This is based on Tony Headford's work.
 * 
 * @author and Copyright (c) for my code Wolfgang Lenerz 2013
 * 
 * Based on code by Tony Headford, see his licence in accompanying file.
 */
public class MOVEQ implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        int base = 0x7000;
        CPU.Instruction i = new CPU.Instruction()
        {
            @Override
            public final void execute(int opcode,CPU.MC68000Cpu cpu)
            {
                int s = opcode & 0xff;
                cpu.reg_sr&=0xfff0;
                if ((s & 0x80)==0x80)
                {
                    s+=0xffffff00;                          // sign extend value
                    cpu.reg_sr+=8;
                }
                else if (s==0)
                {
                    cpu.reg_sr+=4;
                }
                cpu.data_regs[opcode >> 9 & 0x07]= s;
            }
            
            @Override
            public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
            {
                    return disassembleOp(address, opcode);
            }
        };

        for(int reg = 0; reg < 8; reg++)
        {
            for(int imm = 0; imm < 256; imm++)
            {
                cpu2.addInstruction(base + (reg << 9) + imm, i);
            }
        }
    }

    protected final DisassembledInstruction disassembleOp(int address, int opcode)
    {
        DisassembledOperand src = new DisassembledOperand(String.format("#$%02x", opcode & 0xff));
        DisassembledOperand dst = new DisassembledOperand("d" + ((opcode >> 9) & 0x07));

        return new DisassembledInstruction(address, opcode, "moveq", src, dst);
    }
}
