/*
 * Copyright (c) W. Lenerz 2023.
 * This code is subject to the GPL, version 3 or later.
 */
package drivers.ser;

/**
 * Set baud rate for Port(s).
 * 
 * @author and copyright Wolfgang Lenerz
 * @version  1.00 
 */
public class BaudRate
{
     private final static int[]ALLOWED_BAUD_RATES={300,600,1200,2400,4800,9600,19200,38400,57600,115200};
     
     /**
      * Gets the baud rate to be set. If this is somehow wrong, it already sets the CPU error.
      * 
      * @param baud the baud rate wished
      * 
      * @return the baud rate, or -1 if error.
      */
     public static final int getBaudRate(int baud)
     {
        if (baud==0)
            return ALLOWED_BAUD_RATES[ALLOWED_BAUD_RATES.length-1]; // 0 baud requested = highest allowed baud rate
        for (int i=0;i<ALLOWED_BAUD_RATES.length; i++)
        {
            if (baud == ALLOWED_BAUD_RATES[i])
            {
                return baud;
            }
        }
        return -1;
     }
}
