/*
 * Copyright (c) W. Lenerz 2023.
 * This code is subject to the GPL, version 3 or later.
 */
package ui;

/**
 * The config menu in the menu bar, including item action routines.
 * 
 * @author and copyright Wolfgang Lenerz
 * @version  1.00 initial version
 */
public class AboutMenu extends javax.swing.JMenu
{
    /**
     * The menu showing the "About..." information.
     */
    public AboutMenu()
    {
               
        setText("?");
        
        javax.swing.JMenuItem  mit = new javax.swing.JMenuItem(); // show software "about" box
        mit.setText(Localization.Texts[4]);          
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
          showAboutInfo() ;
        });
        this.add(mit);
        
        mit = new javax.swing.JMenuItem();  
        mit.setText(Localization.Texts[152]);                   // show java version
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            showJavaVersion() ;
        });
        this.add(mit);
    }
    
    /**
     * Menu item action : Shows the about menu item.
     * 
     * @param evt ignored.
     */
    private void showAboutInfo() 
    {                                              
        java.net.URL imgURL = MonitorGui.class.getResource("images/combined.png");// get my icon
        if (imgURL!=null)
        {
            javax.swing.ImageIcon image = new javax.swing.ImageIcon(imgURL);    // and make it into correct icon
            javax.swing.JOptionPane.showMessageDialog(this,Localization.Texts[5]+Localization.Texts[106]+ 
                    smsqmulator.SMSQmulator.getCompiledForVersion()+")","SMSQmulator ",1,image);
        }
        else
        {
            javax.swing.JOptionPane.showMessageDialog(this,Localization.Texts[5]+Localization.Texts[106]+ 
                    smsqmulator.SMSQmulator.getCompiledForVersion()+")","SMSQmulator ",1);
        }
    }                        
    
    /**
     * Menu item action : Show java version.
     * 
     * @param evt ignored.
     */
    private void showJavaVersion()                                                   
    {                                                        
        java.net.URL imgURL = MonitorGui.class.getResource("images/combined.png");// get my icon
        String vendor =System.getProperty("java.vendor");
        if (vendor.toUpperCase().equals("N/A"))
            vendor = "unknown (openJDK?)";
        if (imgURL!=null)
        {
            javax.swing.ImageIcon image = new javax.swing.ImageIcon(imgURL);    // and make it into correct icon
            javax.swing.JOptionPane.showMessageDialog(this," Java Version: " + System.getProperty("java.version")+
                           " from "+vendor,"SMSQmulator ",1,image);
        }
        else
        {
            javax.swing.JOptionPane.showMessageDialog(this," Java Version: " + System.getProperty("java.version")+
                           " from "+vendor,"SMSQmulator ",1);
        }
    }
}