/*
 * Copyright (c) W. Lenerz 2023.
 * This code is subject to the GPL, version 3 or later.
 */
package ui;

import drivers.DeviceDriver;
import smsqmulator.TrapDispatcher;

/**
 * The devices menu in the config menu : set up the menu and its assorted item action routines.
 * 
 * @author and copyright Wolfgang Lenerz
 * @version  1.00 initial version
 */
public class DevicesMenu extends javax.swing.JMenu
{
    private final smsqmulator.Monitor monitor;
    private final inifile.IniFile inifile;
    private final MonitorGui mgui;
    
    /**
     * The Devices menu item in the config item in the menu bar.
     * 
     * @param pmonitor  monitor for the emulation thread
     * @param pinifile  the inifile with all configured options.
     * @param mguip     the monitorgui enclosing the menu bar.
     * */
    public DevicesMenu(smsqmulator.Monitor pmonitor,inifile.IniFile pinifile,MonitorGui mguip,TrapDispatcher td)
    {
        this.monitor=pmonitor;
        this.inifile=pinifile;
        this.mgui=mguip;
        initMenu(td);
    }
   
    private void initMenu(smsqmulator.TrapDispatcher td)
    {
        setText(Localization.Texts[132]);
        javax.swing.JMenuItem mit;
        javax.swing.JCheckBoxMenuItem cbox;
        javax.swing.JMenu menu;
        javax.swing.ButtonGroup buttonGroup;
        javax.swing.JRadioButtonMenuItem rbutton;
        
// win drives
        mit = new javax.swing.JMenuItem();                      // setWinNameMenuItem.  win device native filenames
        mit.setText(Localization.Texts[14]);                     
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            String[] oldNames = drivers.win.WinDriver.getNativeNames();
            getAndSetDriveNamesFromDialog(oldNames,"WIN",smsqmulator.Types.WINDriver,false) ;
        });
        this.add(mit);
        
        mit = new javax.swing.JMenuItem();                      // winlUseMenuItem  win device use name
        mit.setText(Localization.Texts[15]);                    
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
           getUsageName("WIN") ;
        });
        this.add(mit);
        
        cbox = new javax.swing.JCheckBoxMenuItem();             // .winDisableCheckBoxMenuItem 
        cbox.setText(Localization.Texts[99]);
        cbox.setSelected(this.inifile.getTrueOrFalse("DISABLE-WIN-DEVICE"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            ConfigData.setOptionAndSave("DISABLE-WIN-DEVICE", cb.isSelected() ?"1":"0",this.inifile);
        });
        this.add(cbox);
        
        cbox = new javax.swing.JCheckBoxMenuItem();             // winIgnoreFilelockErrorCheckBoxMenuItem
        cbox.setText(Localization.Texts[125]);
        cbox.setSelected(this.inifile.getTrueOrFalse("IGNORE-QXLWIN-LOCK-ERROR"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            ConfigData.setOptionAndSave("IGNORE-QXLWIN-LOCK-ERROR", cb.isSelected() ?"1":"0",this.inifile);
   //xxxxx         this.winUnlockable2ReadOnlyCheckBoxMenuItem.setEnabled(this.winIgnoreFilelockErrorCheckBoxMenuItem.isSelected());
        });
        this.add(cbox);
        
        cbox = new javax.swing.JCheckBoxMenuItem();             // winUnlockable2ReadOnlyCheckBoxMenuItem
        cbox.setText(Localization.Texts[144]);
        cbox.setSelected(this.inifile.getTrueOrFalse("MAKE-UNLOCKABLE-QXLWIN-READONLY"));
        cbox.setEnabled(inifile.getTrueOrFalse("IGNORE-QXLWIN-LOCK-ERROR"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            ConfigData.setOptionAndSave("MAKE-UNLOCKABLE-QXLWIN-READONLY", cb.isSelected() ?"1":"0",this.inifile);
        });
        this.add(cbox);
        this.add(new javax.swing.JPopupMenu.Separator());
        
// nfa drives
        mit = new javax.swing.JMenuItem();                      // setNFANameMenuItem  nfa device native dirs
        mit.setText(Localization.Texts[8]);                     
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            String[] oldNames = td.getNamesForDrives(smsqmulator.Types.NFADriver).clone();
            getAndSetDriveNamesFromDialog(oldNames,"NFA",smsqmulator.Types.NFADriver,true) ;
        });
        this.add(mit);
           
        buttonGroup = new javax.swing.ButtonGroup();         
        menu = new javax.swing.JMenu();                         //  changeNfaNamesMenu
        menu.setText(Localization.Texts[8]);   
            rbutton = new javax.swing.JRadioButtonMenuItem();   // don't change
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[21]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                setNfaNameChange(0);
            });
            menu.add(rbutton);
            rbutton = new javax.swing.JRadioButtonMenuItem();   // upper case
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[22]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                setNfaNameChange(1);
            });
            menu.add(rbutton);
            rbutton = new javax.swing.JRadioButtonMenuItem();   // lower case
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[23]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                setNfaNameChange(2);
            });
            menu.add(rbutton);
        this.add(menu);
        
        mit = new javax.swing.JMenuItem();                      // nfaUseMenuItem  nfa device use name
        mit.setText(Localization.Texts[10]);                    
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
           getUsageName("NFA") ;
        });
        this.add(mit);
        
        cbox = new javax.swing.JCheckBoxMenuItem();             // .nfaDisableCheckBoxMenuItem 
        cbox.setText(Localization.Texts[100]);
        cbox.setSelected(this.inifile.getTrueOrFalse("DISABLE-NFA-DEVICE"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            ConfigData.setOptionAndSave("DISABLE-NFA-DEVICE", cb.isSelected() ?"1":"0",this.inifile);
        });
        this.add(cbox);
        
        cbox = new javax.swing.JCheckBoxMenuItem();             // nfa use xtcc item
        cbox.setText(Localization.Texts[153]);
        cbox.setSelected(this.inifile.getTrueOrFalse("USE_XTCC"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            ConfigData.setOptionAndSave("USE_XTCC", cb.isSelected() ?"1":"0",this.inifile);
        });
        this.add(cbox);
        this.add(new javax.swing.JPopupMenu.Separator());
        
// sfa drives
        mit = new javax.swing.JMenuItem();                      // setNFANameMenuItem  nfa device native dirs
        mit.setText(Localization.Texts[11]);                     
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            String[] oldNames = td.getNamesForDrives(smsqmulator.Types.SFADriver).clone();
          //  String[] oldNames = drivers.nxfa.SfaDriver.getNativeNames();
            getAndSetDriveNamesFromDialog(oldNames,"SFA",smsqmulator.Types.SFADriver,true) ;
        });
        this.add(mit);
           
        buttonGroup = new javax.swing.ButtonGroup();         
        menu = new javax.swing.JMenu();                         //  changeSfaNamesMenu
        menu.setText(Localization.Texts[12]);   
            rbutton = new javax.swing.JRadioButtonMenuItem();   // don't change
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[21]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                setSfaNameChange(0);
            });
            menu.add(rbutton);
            rbutton = new javax.swing.JRadioButtonMenuItem();   // upper case
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[22]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                setSfaNameChange(1);
            });
            menu.add(rbutton);
            rbutton = new javax.swing.JRadioButtonMenuItem();   // lower case
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[23]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                setSfaNameChange(2);
            });
            menu.add(rbutton);
        this.add(menu);
        
        mit = new javax.swing.JMenuItem();                      // sfaUseMenuItem  sfa device use name
        mit.setText(Localization.Texts[13]);                    
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
           getUsageName("SFA") ;
        });
        this.add(mit);
        
        cbox = new javax.swing.JCheckBoxMenuItem();             // sfaDisableCheckBoxMenuItem 
        cbox.setText(Localization.Texts[101]);
        cbox.setSelected(this.inifile.getTrueOrFalse("DISABLE-SFA-DEVICE"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            ConfigData.setOptionAndSave("DISABLE-SFA-DEVICE", cb.isSelected() ?"1":"0",this.inifile);
        });
        this.add(cbox);
        this.add(new javax.swing.JPopupMenu.Separator());
        
// flp drives
        mit = new javax.swing.JMenuItem();                      // setflpNameMenuItem  flp device native files
        mit.setText(Localization.Texts[79]);                     
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            String[] oldNames = drivers.FloppyDriver.getNativeNames();
            getAndSetDriveNamesFromDialog(oldNames,"FLP",smsqmulator.Types.FLPDriver,false) ;
        });
        this.add(mit);
           
        mit = new javax.swing.JMenuItem();                      // flpUseMenuItem  flp device use name
        mit.setText(Localization.Texts[80]);                    
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
           getUsageName("FLP") ;
        });
        this.add(mit);
        
        cbox = new javax.swing.JCheckBoxMenuItem();             // flpDisableCheckBoxMenuItem 
        cbox.setText(Localization.Texts[102]);
        cbox.setSelected(this.inifile.getTrueOrFalse("DISABLE-FLP-DEVICE"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            ConfigData.setOptionAndSave("DISABLE-FLP-DEVICE", cb.isSelected() ?"1":"0",this.inifile);
        });
        this.add(cbox);
        this.add(new javax.swing.JPopupMenu.Separator());

// MEM drive        
        mit = new javax.swing.JMenuItem();                      // setMemNamesMenuItem
        mit.setText(Localization.Texts[118]);                     
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            String[] oldNames = drivers.FloppyDriver.getNativeNames();
            getAndSetDriveNamesFromDialog(oldNames,"FLP",smsqmulator.Types.FLPDriver,false) ;
        });
        this.add(mit);
           
        mit = new javax.swing.JMenuItem();                      //  memUseMenuItem  flp device use name
        mit.setText(Localization.Texts[119]);                    
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
           getUsageName("FLP") ;
        });
        this.add(mit);
        
        cbox = new javax.swing.JCheckBoxMenuItem();             //    memDisableCheckBoxMenuItem
        cbox.setText(Localization.Texts[120]);
        cbox.setSelected(this.inifile.getTrueOrFalse("DISABLE-MEM-DEVICE"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            ConfigData.setOptionAndSave("DISABLE-FLP-DEVICE", cb.isSelected() ?"1":"0",this.inifile);
        });
        this.add(cbox);
        this.add(new javax.swing.JPopupMenu.Separator());
        
// ser port
        mit = new javax.swing.JMenuItem();                      //    serPortsMenuItem
        mit.setText(Localization.Texts[156]);    
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            String [] serNames =  new SerAssignmentsDialog(this.mgui,true,ConfigData.getSerPortsData(this.inifile)).getPorts();
            if (serNames!=null)
            {
                this.monitor.getTrapDispatcher().setSerdriverNames(serNames);
                ConfigData.setSerPortsData(serNames,this.inifile);
            }
        });
        this.add(mit);
    }
    
    /**
     * Gets names of dirs/files for win, nfa, sfa, etc from user, sets them and saves them to inifile.
     * 
     * @param oldNames      the array currently holding the names
     * @param deviceName    string name of device, 3 letters, uppercased.
     * @param dType         which device.
     * @param mustBeDir     true if file to be chosen must be a dir.
     * 
     * @return              the new array with (unprocessed) filenames.
     */
    private void getAndSetDriveNamesFromDialog(String[] oldNames,String deviceName,int dType,boolean mustBeDir) 
    {
        String[] newNames = getDriveNamesFromDialog(oldNames,deviceName,dType,mustBeDir) ;
        this.monitor.getTrapDispatcher().setNamesForDrives(dType,drivers.ResolvePath.resolvePaths(newNames,mustBeDir),true);
        ConfigData.storeDriveNames(deviceName,newNames,this.inifile);
    }
    
   
    /**
     * Gets names of dirs/files for win, nfa, sfa, etc from user, save to inifile if any change.
     * 
     * @param oldNames      the array currently holding the names
     * @param deviceName    string name of device, 3 letters, uppercased.
     * @param dType         which device.
     * @param mustBeDir     true if file to be chosen must be a dir.
     * 
     * @return              the new array with (unprocessed) filenames.
     */
    private String[] getDriveNamesFromDialog(String[] oldNames,String deviceName,int dType,boolean mustBeDir) 
    {                                               
        ui.DriveAssignmentsDialog dad=new ui.DriveAssignmentsDialog(null,true,oldNames,deviceName,mustBeDir);// setup dialog, don'y show it yet
        java.awt.Point p = this.getMousePosition();
        if (p==null)
            p=new java.awt.Point();
        java.awt.Point p1= this.mgui.getLocationOnScreen();           // used to calculate where the dialog will appear, i.e. close to the mouse position when clicked
        if (!dad.adoptChanges(oldNames,p.x+p1.x+10,p.y+p1.y+10))
            return oldNames;                                    // user clocked ESC or close, so nothing has changed, return old
        
        return dad.getOptions();
    }
    
    /**
     * Gets the device usage name from the user and sets it.
     * 
     * @param device the device for which to set the usage name
     */
    private void getUsageName(String device)
    {
        device=device.toUpperCase();
        String s=this.inifile.getOptionValue(device+"_USE");    // try to get current use option from ini file
        if (s==null || s.isEmpty())
            s=device;
        s = (String)javax.swing.JOptionPane.showInputDialog(this,Localization.Texts[36]+"\n"+ device +Localization.Texts[37],
                    Localization.Texts[38]+device,javax.swing.JOptionPane.PLAIN_MESSAGE,null,null,s);// get new name from user
        if (s==null || s.isEmpty())
            return;
        if (s.length()!=3)
            smsqmulator.Helper.reportError(Localization.Texts[30], Localization.Texts[39]+s, this.mgui);
        else
           ConfigData.setOptionAndSave(device+"_USE",s.toUpperCase(),this.inifile);
    }
    
    /**
     * Menu item action : Sets whether the names of files should be changed (on the NFA native file system).
     * 
     * @param change the type of change wished : 0 don't change the names, 1 - set names to upper case, 2 =set names to ower case.
     */ 
    private void setNfaNameChange(int change)
    {
        this.monitor.getTrapDispatcher().setFilenameChange(smsqmulator.Types.NFADriver,change);
        ConfigData.setOptionAndSave("NFA-FILENAME-CHANGE", ""+change,this.inifile);
    }
  
    /**
     * Menu item action : Sets whether the names of files should be changed (on the NFA native file system).
     * 
     * @param change the type of change wished : 0 don't change the names, 1 - set names to upper case, 2 =set names to ower case.
     */ 
    private void setSfaNameChange(int change)
    {
        this.monitor.getTrapDispatcher().setFilenameChange(smsqmulator.Types.SFADriver,change);
        ConfigData.setOptionAndSave("SFA-FILENAME-CHANGE", ""+change,this.inifile);
    }   
}