/*
 * Copyright (c) W. Lenerz 2023.
 * This code is subject to the GPL, version 3 or later.
 */
package ui;

/**
 * The file menu in the menu bar, including item action routines.
 * 
 * @author and copyright Wolfgang Lenerz
 * @version  1.00 initial version
 */
public class FileMenu extends javax.swing.JMenu
{
    private final smsqmulator.Monitor monitor;
    private final inifile.IniFile inifile;
    
    public FileMenu(smsqmulator.Monitor pmonitor,inifile.IniFile pinifile)
    {
        this.monitor=pmonitor;
        this.inifile=pinifile;
        javax.swing.JMenuItem mit ;
        this.setText(Localization.Texts[1]);
        
        mit = new javax.swing.JMenuItem();                      // Loads the SMSQE file
        mit.setText(Localization.Texts[6]);                     
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            loadSmsqeFile();
        });
        this.add(mit);
        
        mit = new javax.swing.JMenuItem();                      // Resets the machine
        mit.setText(Localization.Texts[7]);                     
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            this.monitor.reset();
        });
        this.add(mit);
        
        mit = new javax.swing.JMenuItem();                      // exits the program
        mit.setText(Localization.Texts[141]);                     
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            System.exit(0);
        });
        this.add(mit);
    }
    
     /**
     * Item action : Loads the SMSQE file, stops the emulation & re-starts the emulation with the new SMSQE file.
     * 
     * @param evt ignored
     */
    private void loadSmsqeFile() 
    {                                                      
        String fn=this.inifile.getOptionValue("ROM_IMAGE_FILE");
        javax.swing.JFileChooser fc =new javax.swing.JFileChooser();
        if (fn!=null && !fn.isEmpty())
            fc.setCurrentDirectory(new java.io.File(fn));
        if (fc.showOpenDialog(this)==javax.swing.JFileChooser.APPROVE_OPTION)// get a filename
        {
            java.io.File f= fc.getSelectedFile();
            ConfigData.setOptionAndSave("ROM_IMAGE_FILE", f.getAbsolutePath(),this.inifile);
            this.monitor.reset();
        }
    }
}