/*
 * Copyright (c) W. Lenerz 2023.
 * This code is subject to the GPL, version 3 or later.
 */
package ui;

/**
 * Static method to ppen an optionpane and get a valid (positive or 0) integer (int) from it.
 * 
 * @author and copyright Wolfgang Lenerz
 * @version  1.00 initial version
 */
public class GetIntegerInput
{
    /**
     * Open an optionpane and get a valid (positive or 0) integer (int) from it.
     * 
     * @param menu          parent object
     * @param initialValue  initial proposal
     * @param message       message text
     * @param title         title text
     * @param upperLimit    upper limit of int - if this is negative, upper limit will be set to Integer.MAX_VALUE
     * 
     * @return              the (+ve or 0) integer input by user, or -1 if user didn't enter anything
     */
    public static int getIntegerInput(javax.swing.JMenu menu, int initialValue, String message, String title, int upperLimit)
    {
        boolean doLoop=true;
        int param=-1;
        String s = ""+initialValue;
        if (upperLimit<0)
            upperLimit = Integer.MAX_VALUE;
        while (doLoop)                                        // this will loop until user enters a correct value
        { 
            s = (String)javax.swing.JOptionPane.showInputDialog(
                    menu,
                    message,
                    title,
                    javax.swing.JOptionPane.PLAIN_MESSAGE,
                    null,
                    null,
                    s);
            if (s!=null && !s.isEmpty())
            {
                try
                {
                    param=Integer.parseInt(s.trim());
                    if (param>-1 && param<=upperLimit)
                        return param;                           // premature exit!!!!
                }
                catch (NumberFormatException e)
                {
                    javax.swing.JOptionPane.showMessageDialog(menu, s+Localization.Texts[90], Localization.Texts[45],javax.swing.JOptionPane.ERROR_MESSAGE);
                }
            }
            return -1;                                          // user didn't enter anything
        }
        return param;                                           // this is an unreachable statement
    }
}
